#!/usr/bin/python3

import re
import sys
from cmd import Cmd
from steemax import axe
from steemax import axdb
from steemax import axverify
from steemax import axtrans
from steemax import default
from screenlogger.screenlogger import Msg

msg = Msg(default.logfilename,
          default.logpath,
          "")
db = axdb.AXdb(default.dbuser,
               default.dbpass,
               default.dbname)
xverify = axverify.AXverify(screenmodeoverride="verbose")


# Entry point
def run(args=None):
    db.first_time_setup()
    prompt = MyPrompt()
    prompt.prompt = '[steemax]# '
    prompt.cmdloop('\n   ** Welcome to SteemAX ** \n')


class Enter:
    def __init__(self):
        self.privatekey = None
        self.refreshtoken = None
        self.accesstoken = None
        self.username = None
    
    def new_account(self, acct):
        """Prompts user to create an account
        when an account is not found
        """
        answer = input(("{} has not joined SteemAX.\n"
                        + "Would you like to add this "
                        + "user [y/n]? ").format(acct))
        if answer == "y":
            self.key()
            db.add_user(self.username, self.privatekey,
                        self.refreshtoken,
                        self.accesstoken)
            return True
        else:
            return False

    def account_name(self, flag):
        """ Prompt user for their Steemit account name. 
        'flag' indicates entering the account name 
        for an invitee or inviter
        """
        if flag:
            question = 'Your account name @'
        else:
            question = 'Their account name @'
        while True:
            acct = input(question)
            if (not re.match(r'^[a-z0-9\-]+$', acct)
                    or len(acct) == 0
                    or len(acct) > 32):
                msg.message('The account name you entered is '
                            + 'blank or contains invalid characters.')
            else:
                if xverify.steem.check_balances(acct):
                    break
                else:
                    msg.message('Steemit account ' + acct
                                + ' could not be found.')
        return acct

    def memo_id(self, acct):
        """ Prompt user for the unique Memo ID that 
        was generated during the creation of an invite
        """
        while True:
            memoid = input('Memo ID: ')
            if (not re.match(r'^[0-9]+$', memoid)
                    or len(memoid) == 0
                    or len(memoid) > 32):
                msg.message('The Memo ID you entered is '
                            + 'blank or contains invalid characters.')
            else:
                if db.verify_memoid(acct, memoid):
                    break
                else:
                    return False
        return memoid

    def percentage(self, acct):
        """ Prompt user for the percentage of their 
        upvote they wish to exchange. 
        This is always a percentage of the inviter's 
        (Account1) upvote, not the invitee (Account2)
        """
        while True:
            per = input(
                'Percentage of {}\'s upvote (1 to 100): '.format(acct))
            if (not re.match(r'^[0-9]+$', per)
                    or len(per) == 0
                    or len(per) > 3
                    or int(per) < 1
                    or int(per) > 100):
                msg.message('Please only enter a number '
                            + 'between 1 and 100.')
            else:
                break
        return per

    def ratio(self, acct1, acct2, per, flag):
        """ Prompt user for the ratio between 
        accounts for the exchanges. This ratios 
        is expressed as a two digit decimal number
        to two places in proportion to 1, i.e. 
        XX.xx to 1. e.g. 0.01 to 1, 10.10 to 1, 
        1.25 to 1, etc.
        This is always so that X is inviter's 
        (Account1) upvote, to 1 which is the 
        invitee's (Account2) upvote
        """
        while True:
            ratio = input('Enter ratio '
                          + 'as X ({}) to 1 ({}). X is: '.format(acct1, acct2))
            if (not re.match(r'^[0-9\.]+$', ratio)
                    or len(ratio) == 0
                    or len(ratio) > 8
                    or float(ratio) < 0.001
                    or float(ratio) > 1000):
                msg.message('Please enter a one or two '
                            + 'digit number to represent a ratio in '
                            + 'the format x to 1 where x is your '
                            + 'input. Enter a decimal to two places '
                            + 'to represent a ratio less than one. '
                            + 'e.g. 0.05 to 1')
            else:
                if xverify.eligible_votes(acct1, acct2,
                                          per, ratio, flag):
                    break
        return ratio

    def duration(self):
        """ Promt user to enter the 
        duration of the auto exchange 
        in the number of days
        """
        while True:
            dur = input('Duration of exchange in days: ')
            if (not re.match(r'[0-9]', dur)
                    or len(dur) == 0
                    or len(dur) > 3):
                msg.message('Please only enter a number '
                            + 'up to three digits.')
            else:
                break
        return dur

    def key(self):
        """ Prompt user for their private 
        posting key as found in their 
        steemit.com wallet or for their
        SteemConnect Refresh Token as 
        given by the auth_url login
        """
        msg.message(xverify.steem.connect.auth_url())
        while True:
            key = input('Your Private Posting Key or'
                        + ' SteemConnect Refresh Token: ')
            if len(key) < 16:
                msg.error_message('The private posting key you '
                                  + 'entered is too small.')
            elif xverify.steem.verify_key(acctname="", tokenkey=key):
                self.privatekey = xverify.steem.privatekey
                self.refreshtoken = xverify.steem.refreshtoken
                self.accesstoken = xverify.steem.accesstoken
                self.username = xverify.steem.username
                msg.message("Welcome " + self.username)
                break
            else:
                msg.error_message('Could not verify key or token.')
        return self.username


class MyPrompt(Cmd):
    """ Command line interface for SteemAX
    """

    def do_run(self, args):
        """ Runs the auto exchange on all
        eligible accounts
        """
        axe.Axe().exchange()

    def do_process(self, args):
        """ processes transactions for their
        memo messages and responds accordingly
        """
        xtrans = axtrans.AXtrans()
        xtrans.fetch_history()

    def do_expire(self, args):
        """ Sets the status to 4 (cancelled/expired)
        if the timestamp is past datetime.now 
        """
        db.expire()

    def do_adduser(self, args):
        """ Adds a user to steemax
        """
        enter = Enter()
        acct = enter.key()
        db.add_user(acct, enter.privatekey,
                    enter.refreshtoken,
                    enter.accesstoken)

    def do_invite(self, args):
        """ Initiates an invite. If a user
        is not a member it adds them
        """
        enter = Enter()
        acct1 = enter.account_name(1)
        if not db.get_user_token(acct1):
            enter.new_account(acct1)
        acct2 = enter.account_name(0)
        per = enter.percentage(acct1)
        ratio = enter.ratio(acct1, acct2, per, 1)
        dur = enter.duration()
        memoid = db.add_invite(acct1, acct2,
                               per, ratio, dur)
        if memoid:
            msg.message('An invite has been created. To '
                        + 'authorize this exchange and to send '
                        + 'the invite please send any amount of '
                        + 'SBD to @steem-ax along with the following '
                        + 'memo message. Your SBD will be forwarded '
                        + 'to the invitee:\n\n   '
                        + '{}:start'.format(memoid))
        else:
            msg.message("An invite could not be created.")

    def do_barter(self, args):
        """ Barter on the 
        percentage, ratio and 
        duration of an exchange
        """
        acct = Enter().account_name(1)
        memoid = Enter().memo_id(acct)
        if not db.verify_memoid(acct, memoid):
            return False
        msg.message(('{} is the inviter and {} is the invitee.'
                     ).format(db.inviter, db.invitee))
        per = Enter().percentage(db.inviter)
        ratio = Enter().ratio(db.inviter, db.invitee, per, 1)
        dur = Enter().duration()
        msg.message('To initiate this barter send '
                    + 'any amount SBD to @steem-ax with '
                    + 'the following in the memo:\n\n  '
                    + '{}:barter:{}:{}:{}'.format(
                    memoid, per, ratio, dur))

    def do_cancel(self, args):
        """ Cancel an invite to an exchange
        """
        acct = Enter().account_name(1)
        memoid = Enter().memo_id(acct)
        if not db.verify_memoid(acct, memoid):
            return
        if db.cancel(acct, memoid):
            msg.message("The exchange has been canceled")

    def do_eligible(self, args):
        """ Find out if a certain 
        percentage and ratio between 
        two accounts will create an 
        eligible exchange
        """
        acct1 = Enter().account_name(1)
        acct2 = Enter().account_name(0)
        per = Enter().percentage(acct1)
        ratio = Enter().ratio(acct1, acct2, per, 1)
        if xverify.eligible_posts(acct1, acct2) is not False:
            print("Posts are eligible.")
            # In the method eligible_votes
            # the last argument is a flag set to "1" indicating
            # that vote values should be calculated at
            # 100% voting power. If the flag were set to "2"
            # even if the calculated vote weight for the invitee
            # exceeds more than 100%
            # or less than 1% to go ahead and
            # make the exchange anyway at 100% or 1%
            # If the flag is set to 1 vote values are calculated
            # at 100% voting power, whereas if the flag is set
            # to 0 or 2 vote values are calculated at their
            # current voting power.
            if xverify.eligible_votes(acct1,
                                      acct2,
                                      per,
                                      ratio,
                                      0) is not False:
                print("Votes are eligible")

    def do_account(self, args):
        """ Find and verify a Steemit 
        account and see if it has 
        started an exchange
        """
        acct = Enter().account_name(1)
        xverify.get_vote_value(acct)
        msg.message("""
    __{}__
    Vote Power: {}
    Steem Power: {}   
    Vote Value at {}%: ${}
            """.format(acct,
                       xverify.steem.votepower,
                       xverify.steem.steempower,
                       xverify.voteweight,
                       xverify.votevalue))

    def do_list(self, args):
        account = input("Account (press enter for none): ")
        if not account or account == "" or account == 0:
            account = None
        print ("Getting list for " + account)
        axlist = db.get_axlist(account)
        for value in axlist:
            print (value[1] + " vs. " + value[2])
            print ('    MemoID: ' + value[6])
            print ('    Per: ' + value[3]
                   + '%    Ratio: ' + value[4]
                   + ':1      Dur: ' + value[5]
                   + ' days     Status: ' + value[7] + '\n')

    def do_exchanges(self, args):
        """ Lists all the archived exhanges for an account
        """
        account = input("Account (press enter for none): ")
        # If no account is entered return the whole list
        if not account or account == "" or account == 0:
            account = None
        else:
            print ("Looking up " + account)
        axlist = db.get_exchange_archive(account)
        for trade in axlist:
            print(str(trade[6]))
            print("@" + str(trade[0]) + "/" + str(trade[2])
                  + "\nvs.\n" + "@" + str(trade[1]) + "/" + str(trade[3]))
            print(str(trade[4]) + " vs. " + str(trade[5]) + "\n\n")

    def do_pool(self, args):
        """ Display current Steemit Reward Balance, 
        Recent Claims and price of STEEM
        """
        xverify.steem.reward_pool_balances()
        msg.message("""
    ------------------------------------------------
    Reward balance: {}
    Recent claims: {}
    Steem = ${}
    ------------------------------------------------
            """.format(xverify.steem.reward_balance,
                       xverify.steem.recent_claims,
                       xverify.steem.price_of_steem))

    def do_quit(self, args):
        """Quits the program."""
        print ("Quitting.")
        raise SystemExit

    def do_exit(self, args):
        """Quits the program."""
        print ("Quitting.")
        raise SystemExit


# Run as main

if __name__ == "__main__":
    if sys.version_info[0] < 3:
        raise Exception("Python 3 or a more recent version is required.")
    run()

# EOF



#!/usr/bin/python3

import re
import urllib3
import json
from urllib.parse import urlencode
from screenlogger.screenlogger import Msg
from simplesteem.simplesteem import SimpleSteem
from steemax import axdb
from steemax import axverify
from steemax import default
from steemax import sec


class Web:
    def __init__(self):
        self.steem = SimpleSteem(
            client_id=default.client_id,
            client_secret=default.client_secret,
            callback_url=default.callback_url,
            screenmode=default.msgmode)
        self.db = axdb.AXdb(
            default.dbuser,
            default.dbpass,
            default.dbname)
        self.msg = Msg(default.logfilename,
                       default.logpath,
                       default.msgmode)
        self.json_resp = None

    def load_template(self, templatefile=""):
        """ opens a template file and loads it
        into memory stored in a variable
        """
        templatepath = default.webpath + "/" + templatefile
        with open(templatepath, 'r') as fh:
            try:
                template = fh.read()
            except Exception as e:
                self.msg.error_message(e)
                return None
            else:
                return template

    def make_page(self, template, **kwargs):
        """ Fills in key / value pairs on a 
        given template
        """
        regobj = re.compile(
            r"^(.+)(?:\n|\r\n?)((?:(?:\n|\r\n?).+)+)",
            re.MULTILINE)
        newtemplate = regobj.sub('', template)
        for key, value in kwargs.items():
            newtemplate = re.sub(str(key),
                                 str(value),
                                 newtemplate)
        return newtemplate

    def index_page(self):
        """ Creates the Home page for SteemAX and includes
        the number of users and the number of exchanges.
        """
        return ("\r\n"
                + self.make_page(self.load_template(
                    "templates/index.html"),
                    USERS=self.db.number_of_users(),
                    EXCHANGES=self.db.number_of_exchanges()))

    def login(self, token, dest="home"):
        """ logs a user in using SteemConnect
        adds the user to the database if it's
        their first time.
        """
        if self.verify_token(token):
            if self.db.get_user_token(self.steem.username):
                self.db.update_token(self.steem.username,
                                     self.steem.accesstoken,
                                     self.steem.refreshtoken)
            else:
                self.db.add_user(self.steem.username,
                                 self.steem.privatekey,
                                 self.steem.refreshtoken,
                                 self.steem.accesstoken)
            if dest == "info":
                return ("\r\n"
                        + self.info_page(self.steem.username))
            else:
                autocheck = ""
                if self.db.auto_accept(self.steem.username):
                    autocheck = "checked"
                settings = (self.make_page(self.load_template(
                            "templates/settings.html"),
                            AUTOCHECK=autocheck))
                return ("\r\n"
                        + self.make_page(self.load_template(
                            "templates/invite_form.html"),
                            ACCOUNT1=self.steem.username,
                            REFRESHTOKEN=self.steem.refreshtoken,
                            INVITECOUNT=self.db.number_of_invites(self.steem.username),
                            SETTINGS=settings))
        else:
            return self.auth_url()

    def invite(self, token, account2, per,
               ratio, dur, response, ip, ajax, autocheck):
        """ Creates an invite. First we filter the users
        data then we create the invite in the database.
        If the request was sent via ajax then the returned
        result is simply the number one to indicate success.
        """
        ajax = sec.filter_number(int(ajax))
        response = sec.filter_token(response)
        per = sec.filter_number(per)
        ratio = sec.filter_number(ratio, 1000)
        dur = sec.filter_number(dur, 365)
        autocheck = sec.filter_number(autocheck)
        if not self.verify_recaptcha(response, ip):
            return self.error_page("Invalid captcha.", ajax)
        if float(per) < 1 or float(per) > 100:
           return self.error_page("The percentage can not be lower then "
                                + "1 or greater than 100 and must be a whole number.", ajax)    
        if float(dur) < 7 or float(dur) > 365:
           return self.error_page("The duration must be between 7 and 365 days.", ajax)
        if float(ratio) < 0.001 or float(ratio) > 1000:
           return self.error_page("The ratio must be between 0.001 and 1000 to 1.", ajax)
        if self.verify_token(sec.filter_token(token)):
            account2 = sec.filter_account(account2)
            if self.steem.account(account2) is False:
                return self.error_page(account2 
                    + " is an invalid account name.", ajax)
            else:
                verify = axverify.AXverify()
                if verify.eligible_votes(self.steem.username, account2, per, ratio, 1) is False:
                    return self.error_page(verify.response, ajax)
                memoid = self.db.add_invite(self.steem.username, account2, per, ratio, dur)
                self.db.set_settings(self.steem.username, autocheck)
                if memoid is False:
                    return self.error_page(self.db.errmsg, ajax)
                elif int(memoid) > 0:
                    if (int(ajax) == 1):
                        return "\r\n1"
                    else:
                        return ("Location: https://steemax.info/@"
                                + self.steem.username + "\r\n")
        else:
            return self.auth_url()

    def archive_page(self, account=None):
        """ Provides a list of all the exchanges that
        have occured for a particular account. if
        no account is provided then a list of all exchanges
        for all accounts is returned.
        """
        infobox = ""
        if account is not None:
            account = sec.filter_account(account)
            # If the token is invalid return user to the login page
            if not self.db.get_user_token(account):
                return self.auth_url()
        axlist = self.db.get_exchange_archive(account)
        boxtemplate = self.load_template("templates/archivebox.html")
        for trade in axlist:
            date = (str(trade[6].strftime("%B")) + " "
                    + str(trade[6].day) + ", "
                    + str(trade[6].year))
            time = (str(trade[6].hour) + ":"
                    + str(trade[6].minute))
            box = self.make_page(boxtemplate,
                                 DATE=date,
                                 TIME=time,
                                 ACCOUNT1=trade[0],
                                 ACCOUNT2=trade[1],
                                 IDENT1=trade[2],
                                 IDENT2=trade[3],
                                 VOTEVALUE1=trade[4],
                                 VOTEVALUE2=trade[5])
            infobox = infobox + box
        pagetemplate = self.load_template("templates/archive.html")
        # If the account was provide we display it with the @
        # however if it's not we convert it to an empty string
        # so that it is not displayed in HTML
        if account is None:
            account = ""
        else:
            account = "@" + account
        return ("\r\n" + self.make_page(pagetemplate,
                                        ACCOUNT=account,
                                        INFOBOX=infobox))

    def info_page(self, account):
        """ Creates the page at steemax.info that displays
        all the exchanges a user is involved in and which
        provides the options to accept, barter or cancel a 
        particular exchange.
        """
        account = sec.filter_account(account)
        if not self.db.get_user_token(account):
            return self.auth_url()
        axlist = self.db.get_axlist(account)
        boxtemplate = self.load_template("templates/infobox.html")
        infobox = ""
        invitee = 0
        myaccount = ""
        otheraccount = ""
        buttoncode = ""
        exp = ""
        sendall = []
        acceptall = []
        cancelall = []
        for value in axlist:
            if account == value[2]:
                invitee = 1
                otheraccount = value[1]
                myaccount = value[2]
            else:
                invitee = 0
                otheraccount = value[2]
                myaccount = value[1]
            buttoncode = """
                <div class="closemodal-info" onClick="command('{}', '{}', '{}')">&times;</div>
                """.format(
                value[0],
                otheraccount,
                "cancel")
            cancelall.append(value[0])
            if int(value[7]) == -1 and invitee == 0:
                buttoncode += """
                <div class="send-button" onClick="command('{}', '{}', '{}')">Send</div>
                    """.format(
                    value[0],
                    otheraccount,
                    "start")
                sendall.append(value[0])
            if ((int(value[7]) == 0 and invitee == 1)
                    or (int(value[7]) == 2 and invitee == 1)
                    or (int(value[7]) == 3 and invitee == 0)):
                buttoncode += """
                <div class="accept-button" onClick="command('{}', '{}', '{}')">Accept</div>
                <div class="barter-button" onClick="barter_window('{}');">Barter</div>
                    """.format(
                    value[0],
                    otheraccount,
                    "accept",
                    value[0])
                acceptall.append(value[0])
            elif ((int(value[7]) == 0 and invitee == 0)
                    or (int(value[7]) == 3 and invitee == 1)
                    or (int(value[7]) == 2 and invitee == 0)):
                buttoncode += '<div id="pending">Pending</div>\n'
            if int(value[7]) == 1:
                buttoncode += '<div id="pending">Active</div>\n'
            if value[5] > 0:
                if int(value[7]) == 1:
                    exp = "Expires " + self.db.expiration_date(value[8], value[5])
                else:
                    exp = value[5] + " days"
            else:
                exp = "Until canceled"
            if int(value[7]) != 4 and not (int(value[7]) == -1 and invitee == 1):
                box = self.make_page(boxtemplate,
                                    AXID=value[0],
                                    ACCOUNT1=value[1],
                                    ACCOUNT2=value[2],
                                    PERCENTAGE=value[3],
                                    DURATION=exp,
                                    DARATIO=value[4],
                                    MEMOID=value[6],
                                    INVITEE=invitee,
                                    OTHERACCOUNT=otheraccount,
                                    MYACCOUNT=myaccount,
                                    BTNCODE=buttoncode)
                infobox = infobox + box
        pagetemplate = self.load_template("templates/info.html")
        return ("\r\n" + self.make_page(pagetemplate, 
                                    ACCOUNT1=account,
                                    SENDALL=','.join(map(str, sendall)),
                                    CANCELALL=','.join(map(str, cancelall)),
                                    ACCEPTALL=','.join(map(str, acceptall)),
                                    INFOBOX=infobox))

    def verify_token(self, token):
        """ cleans and verifies a SteemConnect
        refresh token
        """
        token = sec.filter_token(token)
        if (token is not None
            and self.steem.verify_key(
                acctname="", tokenkey=token)):
            return True
        else:
            return False

    def auth_url(self):
        """ Returns the SteemConnect authorization
        URL for SteemAX
        """
        url = self.steem.connect.auth_url()
        return "Location: " + url + "\r\n"

    def error_page(self, msg, ajax):
        """ Returns the HTML page with the
        given error message
        """
        if (int(ajax) == 1):
            return "\r\n"+msg
        else:
            msg += ("<br><br>You may be receiving this error in this format "
                + "because you have disabled javascript. Please enable javascript "
                + "for a better user experience.")
            return ("\r\n"
                    + self.make_page(
                        self.load_template("templates/error.html"),
                        ERRORMSG=msg))

    def verify_recaptcha(self, response, remoteip):
        """ Verifies a Google recaptcha v2 token
        """
        http = urllib3.PoolManager()
        encoded_args = urlencode({'secret': default.recaptcha_secret,
                                  'response': response,
                                  'remoteip': remoteip})
        url = default.recaptcha_url + "?" + encoded_args
        req = http.request('POST', url)
        if req.status == 200:
            self.json_resp = json.loads(req.data.decode('utf-8'))
        if self.json_resp['success']:
            return True
        else:
            return False

# EOF



#!/usr/bin/python3

import pymysql
import re
from screenlogger.screenlogger import Msg
from steemax import default


class DB:
    def __init__(self, dbuser, dbpass, dbname):
        self.dbuser = dbuser
        self.dbpass = dbpass
        self.dbname = dbname
        self.msg = Msg(default.logfilename,
                       default.logpath,
                       default.msgmode)
        self.db = None
        self.cursor = None
        self.dbresults = none

    def open_db(self):
        """ opens a database connection
        """
        self.db = pymysql.connect("localhost",
                                  self.dbuser,
                                  self.dbpass,
                                  self.dbname)
        self.cursor = self.db.cursor()

    def get_results(self, sql, *args):
        """ Gets the results of an SQL statement
        """
        self.open_db()
        try:
            self.cursor.execute(pymysql.escape_string(sql), args)
            self.dbresults = self.cursor.fetchall()
        except Exception as e:
            self.msg.error_message(e)
            self.dbresults = False
            self.db.rollback()
            return False
        else:
            return len(self.dbresults)
        finally:
            # Every call to this method from the other
            # classes in this package only takes place once
            # so we can safely close the DB every time
            # This can be moved to the deconstructor
            # if functionality demands multiple calls to
            # this method
            self.db.close()

    def commit(self, sql, *args):
        """ Commits the actions of an SQL
        statement to the database
        """
        self.open_db()
        try:
            self.cursor.execute(pymysql.escape_string(sql), args)
            self.db.commit()
        except Exception as e:
            self.msg.error_message(e)
            self.db.rollback()
            return False
        else:
            return True
        finally:
            self.db.close()

# EOF


#!/usr/bin/python3

import sys
import re
from dateutil.parser import parse
from datetime import datetime
from steemax import axdb
from steemax import axverify
from steemax import default
from steemax import sec
from screenlogger.screenlogger import Msg
from simplesteem.simplesteem import SimpleSteem


class MemoMsg:
    def __init__(self):
        self.db = axdb.AXdb(default.dbuser,
                            default.dbpass,
                            default.dbname)
        self.reaction = None
        self.returnmsg = None
        self.memoid = None
        self.action = None
        self.percentage = None
        self.ratio = None
        self.duration = None
        self.sendto = None

    def invite_msg(self, acct1, acct2,
                   memoid):
        """ Creates the invite message
        """
        self.reaction = "started"
        self.db.update_status(0, memoid)
        invite = self.db.get_invite(memoid)
        msg = ("Hello @{}, @{} has invited you "
               + " to an auto-upvote exchange. "
               + "They have offered {}% of their upvote "
               + "at a ratio of {}:1 for {} days. "
               ).format(acct2, acct1,
                        invite[0], invite[1], invite[2])
        # Verify the user has an account. If they do,
        # send them the memo id code
        if self.db.get_user_token(acct2):
            msg += ("Please send any amount SBD to "
                    + "@steem-ax with the following in "
                    + "the memo: '{}:accept'. ").format(memoid)
        # Otherwise we skip that part of the message. 
        msg += ("For more info please visit "
                + "https://steemax.info/@{}").format(acct2)
        self.returnmsg = msg

    def accepted_msg(self, acct, memoid):
        """ Creates the accept message 
        """
        self.reaction = "accepted"
        self.db.update_status(1, memoid)
        self.returnmsg = (
            "@{} has accepted the invite. ".format(acct)
            + "The auto-upvote exchange will "
            + "begin immediately.")

    def barter_msg(self, reaction, status, acct,
                   per, ratio, dur, memoid, acct2):
        """ Creates the barter message
        """
        self.reaction = reaction
        # We must choose the proper pronun because the percentage
        # is always taken from the inviter's (not invitee) upvote 
        if status == 2:
            pronoun = "their"
        else:
            pronoun = "your"
        # Add the new values to the database
        self.db.update_invite(per, ratio,
                              dur, memoid, status)
        self.returnmsg = (
            ("@{} has offered to barter. They offer "
             + "{}% of {} upvote at a ratio of "
             + "{}:1 for {} days. "
             + "Send any SBD along with this "
             + "memo code to @steem-ax to accept this invite: "
             + "\n\n'{}:accept'. "
             + "To generate your own barter offer, "
             + "please visit https://steemax.info/@{}").format(
                acct, per, pronoun, ratio, dur, memoid, acct2))

    def ignore(self, reason):
        """ Creates the ignore message 
        """
        self.reaction = "refund"
        self.returnmsg = reason

    def cancel(self, canceller, memoid, rstatus, acct2):
        """ The cancel command can be 
        given at any time by either party
        """
        if int(rstatus) == 4:
            self.ignore("The exchange has already been canceled.")
        elif int(rstatus) == -1:
            self.reaction = "refund"
            self.db.update_status(4, memoid)
            self.returnmsg = ("The exchange between you and "
                            + acct2 + " has been canceled.")
        else:
            self.reaction = "cancelled"
            self.db.update_status(4, memoid)
            self.returnmsg = ("@" + canceller
                            + " has canceled the exchange.")

class Reaction(MemoMsg):
    def start(self, acct1, acct2, memofrom,
              rstatus, memoid, ratio):
        """ The start method grants the authority to the 
        exchange from the inviter. 
        """
        # Only the inviter can start the exchange
        if acct1 == memofrom:
            # if the rstatus is -1 then the exchange invite has been
            # created in the database but not yet sent to the invitee.
            if self.db.auto_accept(acct2) and ratio == 1:
                # The accept message is sent
                self.accepted_msg(acct2, memoid)
                self.reaction = "refund"
            if rstatus < 0:
                self.invite_msg(acct1, acct2, memoid)
            else:
                self.ignore("@{} has already".format(acct1)
                            + " started this exchange.")
        # If the invitee sent the memo we tell them to wait.
        elif acct2 == memofrom:
            self.ignore("Invalid action. @{} ".format(acct1)
                        + " has not yet started this "
                        + "exchange.")
        # If it's neither party the memo is invalid.
        else:
            self.ignore("Invalid Memo ID.")

    def accept(self, acct1, acct2, memofrom,
               rstatus, memoid):
        """ The accept method is used to authorize 
        a change whenever an invite or a barter is 
        sent. 
        """
        # The memo is from the inviter
        if acct1 == memofrom:
            # The invite is pending
            # on the inviter to accept a barter
            if int(rstatus) == 3 :
                # The accept message is sent
                self.accepted_msg(acct1, memoid)
            else:
                self.ignore("Invalid command. This invite is not waiting to be accepted.")
        # The memo is from the invitee
        elif acct2 == memofrom:
            # The invite has been initially sent
            # (rstatus = 0) or the invite is pending on the
            # invitee to accept a barter (rstatus = 2)
            if int(rstatus) == 0 or int(rstatus) == 2:
                # The accept message is sent
                self.accepted_msg(acct2, memoid)
            else:
                self.ignore("Invalid command. This invite is not waiting to be accepted.")
        # If the memo is from neither party it is invalid.              
        else:
            self.ignore("Invalid Memo ID.")

    def barter(self, acct1, acct2, memoid, memofrom,
               rstatus, per, ratio, dur):
        """ A barter command can be used any 
        time after both parties have authorized 
        the exchange.
         2 = inviter (account1) has offered to barter
         3 = invitee (account2) has offered to barter
        When a user enter values from the web interface
        the javascript corrects most errors. Here we
        catch errors if the user has decided to create
        their own memo id commands to send directly from
        their wallet and has made a mistake.
        """
        invalid = False
        verify = axverify.AXverify()
        # First we verify that the percentage and ratio
        # create a valid exchange. There are some instances
        # where the values entered, even when within the correct
        # limits, are not possible. This is partly due to vote weight
        # not being alloed to be lower than 1%
        if not verify.eligible_votes(
                acct1, acct2, per, ratio, 1):
            # The eligible_votes method returns an explanation
            # for why the ratio and percentage were invalid
            # usually because the ratio is too high or low
            self.ignore("Barter invalid. " + verify.response)
            return
        # The memo is from the inviter
        if acct1 == memofrom:
            # The invitee has sent a barter offer.
            if int(rstatus) == 3:
                # A barter offer is sent in return
                # to the invitee's barter offer
                self.barter_msg("acct1 bartering", 2,
                                acct1, per, ratio,
                                dur, memoid, acct2)
            else:
                invalid = True
        # The memo is from the invitee
        elif acct2 == memofrom:
            # The invite has been initially sent
            # (rstatus = 0) or the invite is pending on the
            # invitee to accept a barter (rstatus = 2)
            if int(rstatus) == 0 or int(rstatus) == 2:
                # a barter offer is sent in return
                self.barter_msg("acct2 bartering", 3,
                                acct2, per, ratio,
                                dur, memoid, acct1)
            else:
                invalid = True
        # The memo id matches neither party
        else:
            self.ignore("Invalid Memo ID.")
        # The barter offer was sent by the wrong party.
        if invalid:
            self.ignore("Invalid barter. Please be sure that @"
                        + acct1 + " has started the invite. "
                        + "If you've already sent a barter "
                        + "please wait for the other side to "
                        + "respond.")

class AXtrans:
    def __init__(self):
        self.msg = Msg(default.logfilename,
                       default.logpath,
                       default.msgmode)
        self.db = axdb.AXdb(default.dbuser,
                            default.dbpass,
                            default.dbname)
        self.steem = SimpleSteem()
        self.react = Reaction()
        self.memolist = []
        self.sendto = ""
        self.memofrom = ""

    def parse_memo(self, memo=None):
        """ Parses the memo message in a transaction
        for the appropriate action.
        """
        memodict = {}
        memos = []
        self.memolist = []
        if re.search(r'_', memo): 
            try:
                memos = memo.split("_")
            # A broad exception is used because any exception
            # should return false.
            except:
                self.msg.error_message("Unable to parse memo. Error code 1.")
                return False
        else:
            memos.append(memo)
        for i in range(0, len(memos)):
            if i == len(memos):
                break
            try:
                memo_parts = memos[i].split(":")
            # A broad exception is used because any exception
            # should return false.
            except:
                self.msg.error_message("Unable to parse memo. Error code 2.")
                return False
            # Everything from the outside world is 
            # filtered for security and added to a dictionary
            memodict['memoid'] = sec.filter_token(memo_parts[0])
            if len(memo_parts) == 2:
                memodict['action'] = sec.filter_account(memo_parts[1])
            elif len(memo_parts) == 5:
                memodict['action'] = sec.filter_account(memo_parts[1])
                memodict['percentage'] = sec.filter_number(memo_parts[2])
                memodict['ratio'] = sec.filter_number(memo_parts[3], 1000)
                memodict['duration'] = sec.filter_number(memo_parts[4], 365)
            else:
                self.msg.error_message("Unable to parse memo. Error code 3.")
                return False
            # The dictionary is added to a list
            self.memolist.append(memodict.copy())
        return True

    def send(self, to="artopium",
             amt="0.001 SBD",
             msg="test"):
        """ Sends the forwarded amount of SBD along
        with the reaction message
        """
        # Split "SBD" from the amount
        r = amt.split(" ")
        # Actually transfer it
        if self.steem.transfer_funds(to, float(r[0]),
                                     r[1], msg):
            self.msg.message(("Transaction committed. Sent {} "
                              "{} to {} with the memo: "
                              "{}").format(r[0], r[1], to, msg))

    def act(self, acct1, acct2, rstatus, sendto, i):
        """ Decides how to react based on the action
        present in the memo message
        """
        if not self.db.get_user_token(self.memofrom):
            self.react.ignore(
                ("@{} is not a current member of "
                 + " https://steemax.trade !!Join now!! "
                 + "using SteemConnect.").format(self.memofrom))
        elif self.memolist[i]['action'] == "start":
            self.react.start(acct1,
                             acct2,
                             self.memofrom,
                             rstatus,
                             self.memolist[i]['memoid'],
                             self.memolist[i]['ratio'])
        elif self.memolist[i]['action'] == "cancel":
            self.react.cancel(self.memofrom, self.memolist[i]['memoid'], rstatus, acct2)
        elif self.memolist[i]['action'] == "accept":
            self.react.accept(acct1, acct2, self.memofrom, rstatus, self.memolist[i]['memoid'])
        elif self.memolist[i]['action'] == "barter":
            self.react.barter(acct1, acct2, self.memolist[i]['memoid'], self.memofrom, rstatus,
                              self.memolist[i]['percentage'], self.memolist[i]['ratio'], self.memolist[i]['duration'])
        else:
            self.react.ignore("Invalid action.")
        if self.react.reaction == "refund":
            self.sendto = self.memofrom
        else:
            self.sendto = sendto

    def parse_history_record(self, record, lasttrans):
        """ Parses the blockchain record for transdactions
        sent to @steem-ax 
        """
        if (record[1]['op'][0] == 'transfer'
            and datetime.strptime(
                record[1]['timestamp'],
                '%Y-%m-%dT%H:%M:%S')
                > lasttrans
            and re.match(r'^[0-9]{32}',
                         record[1]['op'][1]['memo'])
            and record[1]['op'][1]['to']
                == self.steem.mainaccount):
            return True
        else:
            return False

    def fetch_history(self):
        """ Processes the transaction history of @steem-ax. 
        The memo message is parsed for a valid 
        command and performs the 
        directed action.
        """
        rt = self.db.get_most_recent_trans()
        for a in self.steem.get_my_history():
            # We check to see if it's a new transaction. If not we
            # do nothing.
            if self.parse_history_record(a, rt):
                self.memofrom = a[1]['op'][1]['from']
                r = a[1]['op'][1]['amount'].split(" ")
                received_amount = r[0]
                trxid = a[1]['trx_id']
                txtime = a[1]['timestamp']
                # If so we parse it. multiple actions are parsed into
                # self.memolist
                if self.parse_memo(memo=a[1]['op'][1]['memo']):
                    # Make sure enough SBD was sent to cover sending
                    # multiple transactions.
                    if float(received_amount) * 1000 >= len(self.memolist):
                        # We divide the amount sent by the number of messages
                        # that must be sent.
                        send_amount = int((float(received_amount) * 1000) / len(self.memolist)) / 1000
                        # self.parse_memo creates self.memolist which we 
                        # then iterate through.
                        for i in range(0, len(self.memolist)):
                            if self.db.verify_memoid(self.memofrom, self.memolist[i]['memoid']):
                                # After verifying the memoid matches the account the 
                                # memo was sent from, the verify_memoid method
                                # returns account1, account2, 
                                # and the current transaction status. It also sets the 
                                # "sendto" account. This info is sent to self.act where
                                # it is parsed further based on action.
                                self.act(self.db.dbresults[0][0],
                                         self.db.dbresults[0][1],
                                         int(self.db.dbresults[0][2]),
                                         self.db.sendto, i)
                            else:
                                self.react.ignore("Invalid Memo ID.")
                                self.sendto = self.memofrom
                            print("Sending $"+str(send_amount) + " SBD to " + self.sendto  + " because "+self.react.returnmsg)
                            self.send(self.sendto,
                                      str(send_amount)+" SBD",
                                      self.react.returnmsg)
                            self.db.add_trans(trxid,
                                              self.memofrom,
                                              str(send_amount)+" SBD",
                                              self.memolist[i]['memoid'],
                                              self.react.reaction,
                                              txtime)
                            print(self.react.returnmsg)
                    else:
                        self.react.ignore("Insufficient funds. Please send at least $" 
                                        + str(len(self.memolist)/1000) 
                                        + " SBD to send these invites.")
                        self.sendto = a[1]['op'][1]['from']
                        self.send(self.sendto,
                                  str(received_amount)+" SBD",
                                  self.react.returnmsg)
                        self.db.add_trans(trxid,
                                          self.memofrom,
                                          str(received_amount)+" SBD",
                                          a[1]['op'][1]['memo'],
                                          self.react.reaction,
                                          txtime)
                        print(self.react.returnmsg)
                else:
                    self.react.ignore("Invalid Memo: "+a[1]['op'][1]['memo'])
                    self.sendto = a[1]['op'][1]['from']
                    self.send(self.sendto,
                              str(received_amount)+" SBD",
                              self.react.returnmsg)
                    self.db.add_trans(trxid,
                                      self.memofrom,
                                      str(received_amount)+" SBD",
                                      a[1]['op'][1]['memo'],
                                      self.react.reaction,
                                      txtime)
                    print(self.react.returnmsg)

# EOF



#!/usr/bin/python3
dbuser = "steemax"
dbpass = "SteemAX_pass23"
dbname = "steemax"
msgmode = "verbose"
logpath = ""
logfilename = "steemax.log"
webpath = ""
recaptcha_url = "https://www.google.com/recaptcha/api/siteverify"
recaptcha_secret = ""
client_id = ""
client_secret = ""
callback_url = ""



from setuptools import setup, find_packages

setup(
    python_requires='>=3.0',
    name='SteemAX',
    version='1.5.1',
    packages=['steemax'],
    license='MIT',
    keywords='steemit steem upvote exchange',
    url='http://github.com/artolabs/steemax',
    author='ArtoLabs',
    author_email='artopium@gmail.com',
    install_requires=[
        'python-dateutil',
        'simplesteem',
        'screenlogger',
    ],
    py_modules=['steemax'],
    entry_points = {
        'console_scripts': [
            'steemax=steemax.steemax:run',
        ],
    },
    include_package_data=True,
    zip_safe=False
)
